package choco;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.solver.Solution;
import org.chocosolver.solver.Solver;
import static org.chocosolver.solver.search.strategy.Search.minDomLBSearch;



public class langford {

	public static void main(String[] args) {
		int n_numbers = 9; // 3, 9
		int n_repeats = 3; // 2, 3
		int n = n_numbers*n_repeats;
		Model m = new Model("(" + n_numbers + "," + n_repeats + ")-langford problem");
		IntVar[] seed = m.intVarArray("S", n_numbers+1, 0, n-1);
		IntVar[] vals = m.intVarArray("V", n_numbers+1, 1, n_numbers);
		IntVar[] lang = m.intVarArray("L", n, 1, n_numbers);
		
		// vals[v] = v; needed for the element constraint below
		for(int v  = 1; v <= n_numbers; v++){
			m.arithm(vals[v], "=", v).post();
		}
		
		// seeds are all different
		m.allDifferent(seed).post();
		
		// seeds must be feasible
		for(int s = 1; s <= n_numbers; s++){
			int max_gap = (s+1) * (n_repeats-1);
			int max_position = n - max_gap;
			m.arithm(seed[s], "<=", max_position).post();
		}
		
		// the ruler must be filled	
		for(int s = 1; s <= n_numbers; s++){
			for(int r = 1; r <= n_repeats; r++){
				int offset = (s+1)*(r-1);
				m.element(vals[s], lang, seed[s], -offset).post();
			}
		}

		Solver s = m.getSolver();
		
		// the seeds determine the problem solution.
		// Hence, only search in the seeds!
		s.setSearch(minDomLBSearch(seed));
		
		Solution solution = s.findSolution();
		
		if(solution != null){
			//System.out.println(solution.toString());
			System.out.println("seeds:");
			for (int x = 1; x <= n_numbers; x++)			
				System.out.print(" " + seed[x].getValue());
			System.out.println();
			System.out.println("langford numbers:");
			for (int i = 0; i < n; i++)
				System.out.print(" " + lang[i].getValue());
			System.out.println();
		} else {
			System.out.println("no solutions");
		}
		
		System.out.println();
		System.out.println(" --- Statistics of " + s.getModelName() + ":");
		System.out.println("       number of failures = " + String.valueOf(s.getFailCount()));
		System.out.println("     number of backtracks = " + String.valueOf(s.getBackTrackCount()));
		System.out.println("    elapsed cpu time (ms) = " + String.valueOf(s.getTimeCount()*1000));
	}
}